@@ -2,7 +2,8 @@ package Pisg;
 
 # Documentation(POD) for this module is found at the end of the file.
 
-# Copyright (C) 2001-2002  <Morten Brix Pedersen> - morten@wtf.dk
+# Copyright (C) 2001-2005  <Morten Brix Pedersen> - morten@wtf.dk
+# Copyright (C) 2003-2005  Christoph Berg <cb@df7cb.de>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -102,13 +103,15 @@ sub get_default_config_settings
         defaultpic => '',
         logdir => [],
         nfiles => 0,
-        lang => 'en',
+        lang => 'EN',
         langfile => 'lang.txt',
         cssdir => 'layout/',
         colorscheme => 'default',
+        altcolorscheme => 'none',
         logprefix => '',
         logsuffix => '',
         silent => 0,
+        cachedir => '',
         userpics => 'y',
 
         # Colors / Layout
@@ -137,11 +140,12 @@ sub get_default_config_settings
         showtopics => 1,
         showlinetime => 0,
         showwordtime => 0,
+        showlines => 1,
         showtime => 1,
         showwords => 0,
         showwpl => 0,
         showcpl => 0,
-        showlastseen => 0,
+        showlastseen => 1,
         showlegend => 1,
         showkickline => 1,
         showactionline => 1,
@@ -152,13 +156,16 @@ sub get_default_config_settings
         showrandquote => 1,
         showmuw => 1,
         showmrn => 1,
+        showkarma => 0,
         showmru => 1,
+        showcharts => 0,
         showops => 1,
         showvoices => 0,
         showhalfops => 0,
         showmostnicks => 0,
+        showactivegenders => 0,
         showmostactivebyhour => 0,
-        showmostactivebyhourgraph => 0,
+        showmostactivebyhourgraph => 1,
         showonlytop => 0,
 
         # Less important things
@@ -169,20 +176,23 @@ sub get_default_config_settings
         quotewidth => 80,
         bignumbersthreshold => 'sqrt',
         wordlength => 5,
+        dailyactivity => 0,
         activenicks => 25,
         activenicks2 => 30,
         activenicksbyhour => 10,
         topichistory => 3,
         urlhistory => 5,
+        chartshistory => 5,
         nickhistory => 5,
+        karmahistory => 5,
         wordhistory => 10,
         mostnickshistory => 5,
         mostnicksverbose => 1,
+        nicklimit => 10,
         nicktracking => 0,
         charset => 'iso-8859-1',
         logcharset => '',
         logcharsetfallback => '',
-        irciinick => 'maintainer',
 
         # sorting
         sortbywords => 0,
@@ -191,15 +201,17 @@ sub get_default_config_settings
 
         foulwords => 'ass fuck bitch shit scheisse scheie kacke arsch ficker ficken schlampe',
         violentwords => 'slaps beats smacks',
+        chartsregexp => '(?:np:|(?:now )?playing:? (?:MPEG stream from)?)\s*(.*)',
         ignorewords => '',
         noignoredquotes => 0,
-        tablewidth => 614,
+        tablewidth => 574,
         regexpaliases => 0,
 
         botnicks => '',            # Needed for DCpp format (non-irc)
 
-        dailyactivity => 0,
-        version => "0.62",
+        statsdump => '',           # Debug option
+
+        version => "0.65",
     };
 
     # This enables us to use the search_path in other modules
@@ -225,14 +237,15 @@ sub get_language_templates
 
         if ($line =~ /<lang name=\"([^"]+)\"(?: charset=\"(.*)\")?>/i) {
             # Found start tag, setting the current language
-            my $current_lang = lc($1);
+            my $current_lang = uc($1);
             $self->{tmps}->{$current_lang}{lang_charset} = lc($2);
 
             while (<FILE>) {
                 last if ($_ =~ /<\/lang>/i);
 
                 # Get 'template = "Text"' in language file:
-                if ($_ =~ /(\w+)\s+=\s+"(.*)"\s*$/) {
+                if ($_ =~ /^(\w+)\s*=\s*"(.*)"\s*$/) {
+                    warn "duplicate translation $1 -> $2" if $self->{tmps}->{$current_lang}{$1};
                     $self->{tmps}->{$current_lang}{$1} = $2;
                 }
             }
@@ -295,7 +308,7 @@ sub init_config
                 add_ignore($nick);
             }
 
-            if ($line =~ /sex=(["'])([MmFf])\1/) {
+            if ($line =~ /sex=(["'])([MmFfBb])\1/) {
                 $self->{users}->{sex}{$nick} = lc($2);
             }
         } elsif ($line =~ /<link(.*)>/) {
@@ -356,7 +369,7 @@ sub init_config
 
                     }
                 } elsif ($_ !~ /^$/) {
-                    print STDERR "Warning: $self->{cfg}->{configfile}, line $.: Unrecognized line\n";
+                    print STDERR "Warning: $self->{cfg}->{configfile}, line $.: Unrecognized line: $_";
                 }
             }
         } elsif ($line =~ /<include\s*=\s*(["'])(.+?)\1\s*>/) {
@@ -377,7 +390,7 @@ sub init_config
             print STDERR "Warning: $self->{cfg}->{configfile}, line $.: Missing end on element <$1 (probably multi-line?)\n";
         } elsif ($line =~ /\S/) {
             $line =~ s/\n//;
-            print STDERR "Warning: $self->{cfg}->{configfile}, line $.: Unrecognized line\n";
+            print STDERR "Warning: $self->{cfg}->{configfile}, line $.: Unrecognized line: $line\n";
         }
     }
 
@@ -401,6 +414,9 @@ sub init_pisg
     }
     $self->{cfg}->{timestamp} = $timestamp;
 
+    $self->{cfg}->{lang} = uc $self->{cfg}->{lang};
+    $self->{cfg}->{lang} =~ s/-/_/g; # PT_BR was called PT-BR before
+
     # convert wordlists
     $self->{cfg}->{foulwords} = wordlist_regexp($self->{cfg}->{foulwords}, $self->{cfg}->{regexpaliases});
     $self->{cfg}->{ignorewords} = wordlist_regexp($self->{cfg}->{ignorewords}, $self->{cfg}->{regexpaliases});
@@ -421,10 +437,12 @@ sub init_pisg
         print STDERR "Warning: $self->{cfg}->{configfile}, line $.: Invalid UserPics setting\n";
     }
 
-    print "Using language template: $self->{cfg}->{lang}\n\n" if ($self->{cfg}->{lang} ne 'EN' && !$self->{cfg}->{silent});
-
-    print "Statistics for channel $self->{cfg}->{channel} \@ $self->{cfg}->{network} by $self->{cfg}->{maintainer}\n\n"
-        unless ($self->{cfg}->{silent});
+    unless ($self->{cfg}->{silent}) {
+        print "Statistics for channel $self->{cfg}->{channel} \@ $self->{cfg}->{network} by $self->{cfg}->{maintainer}\n\n";
+        die sprintf "No such language: %s\n", $self->{cfg}->{lang}
+            unless $self->{tmps}->{$self->{cfg}->{lang}};
+        print "Using language template: $self->{cfg}->{lang}\n\n" if $self->{cfg}->{lang} ne 'EN';
+    }
 }
 
 sub do_channel
@@ -448,7 +466,7 @@ sub do_channel
         eval <<_END;
 use Pisg::Parser::$self->{cfg}->{logtype};
 \$analyzer = new Pisg::Parser::$self->{cfg}->{logtype}(
-    cfg => \$self->{cfg}
+    { cfg => \$self->{cfg}, users => \$self->{users} }
 );
 _END
         if ($@) {
@@ -457,7 +475,7 @@ _END
         }
 
         my $stats = $analyzer->analyze();
-        $self->{cfg}->{analyzer} = $analyzer;
+        $self->{cfg}->{analyzer} = $analyzer; # we need the parser in _format_line
 
         # Initialize HTMLGenerator object
         my $generator;
@@ -480,8 +498,12 @@ _END
         if (defined $stats) {
             if ($stats->{parsedlines} > 0) {
                 $generator->create_output();
-            } elsif ($stats->{parsedlines} == 0) {
-                print STDERR "No parseable lines found in logfile ($stats->{totallines} total lines). Skipping.\nYou might be using the wrong format.\n";
+            } else {
+                print STDERR <<_END unless $self->{cfg}->{silent};
+No parseable lines found in logfile ($stats->{totallines} total lines). Skipping.
+-> You might be using the wrong format.
+-> A common error is that the logs do not contain timestamps for each line.
+_END
             }
         }
 
