@@ -1,5 +1,24 @@
 package Pisg::Common;
 
+# pisg - Perl IRC Statistics Generator
+#
+# Copyright (C) 2001-2005  <Morten Brix Pedersen> - morten@wtf.dk
+# Copyright (C) 2003-2005  Christoph Berg <cb@df7cb.de>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
 =head1 NAME
 
 Pisg::Common - some common functions of pisg.
@@ -191,13 +210,12 @@ sub urlencode
 
 sub randomglob
 {
-    my $pattern = shift;
-    my $globpath = shift;
+    my ($pattern, $globpath, $nick) = @_;
     return $pattern unless $pattern =~ /[*?]/;
     my @globs = glob $globpath . $pattern;
     my $return = $globs[int(rand(@globs))];
     unless($return) {
-        print STDERR "Warning: no picture for $pattern found in $globpath\n";
+        print STDERR "Warning: picture $globpath$pattern for $nick not found\n";
         return $pattern;
     }
     $return =~ s/^$globpath//;
