@@ -27,7 +27,7 @@ sub new
     require Pisg::Common;
     Pisg::Common->import();
 
-    my $lang_charset = $self->{tmps}->{lc($self->{cfg}->{lang})}{lang_charset};
+    my $lang_charset = $self->{tmps}->{$self->{cfg}->{lang}}{lang_charset};
     if($lang_charset and $lang_charset ne $self->{cfg}->{charset}) {
         if($have_iconv) {
             # convert from template charset to our
@@ -65,6 +65,9 @@ sub create_output
     open (OUTPUT, "> $fname") or
         die("$0: Unable to open outputfile($fname): $!\n");
 
+    if ($self->{cfg}->{showlines}) {
+        $self->{cfg}->{tablewidth} += 40;
+    }
     if ($self->{cfg}->{showtime}) {
         $self->{cfg}->{tablewidth} += 40;
     }
@@ -126,6 +129,10 @@ sub create_output
         $self->_mostnicks();
     }
 
+    if ($self->{cfg}->{showactivegenders}) {
+        $self->_activegenders();
+    }
+
     if ($self->{cfg}->{showmuw}) {
         $self->_mostusedword();
     }
@@ -134,10 +141,18 @@ sub create_output
         $self->_mostreferencednicks();
     }
 
+    if ($self->{cfg}->{showkarma}) {
+        $self->_karma();
+    }
+
     if ($self->{cfg}->{showmru}) {
         $self->_mosturls();
     }
 
+    if ($self->{cfg}->{showcharts}) {
+        $self->_charts();
+    }
+
     if ($self->{cfg}->{showbignumbers}) {
         $self->_headline($self->_template_text('othernumtopic'));
         _html("<table width=\"$self->{cfg}->{tablewidth}\">\n"); # Needed for sections
@@ -184,18 +199,26 @@ sub _htmlheader
     );
 
     my $CSS;
-    if($self->{cfg}->{colorscheme} =~ /[^\w]/) { # use external CSS file
-        $CSS = "<link rel=\"stylesheet\" type=\"text/css\" href=\"$self->{cfg}->{colorscheme}\">";
+    if($self->{cfg}->{colorscheme} =~ /([^\/.]+)\.[^\/]+$/) { # use external CSS file
+        $CSS = "<link rel=\"stylesheet\" type=\"text/css\" title=\"$1\" href=\"$self->{cfg}->{colorscheme}\">";
     } elsif($self->{cfg}->{colorscheme} ne "none") { # read the chosen CSS file
         my $css_file = $self->{cfg}->{cssdir} . $self->{cfg}->{colorscheme} . ".css";
         open(FILE, $css_file) or open (FILE, $self->{cfg}->{search_path} . "/$css_file") or die("$0: Unable to open stylesheet $css_file: $!\n");
         {
             local $/; # enable "slurp" mode
-            $CSS = "<style type=\"text/css\">\n". <FILE>. "</style>";
+            $CSS = "<style type=\"text/css\" title=\"$self->{cfg}->{colorscheme}\">\n". <FILE>. "</style>";
         }
         close FILE;
     }
 
+    # use alternate CSS file
+    if($self->{cfg}->{altcolorscheme} ne "none" and $self->{cfg}->{altcolorscheme} =~ /[^\w]/) {
+        foreach (split /\s+/, $self->{cfg}->{altcolorscheme}) {
+            /([^\/.]+)\.[^\/]+$/;
+            $CSS .= "\n<link rel=\"alternate stylesheet\" type=\"text/css\" title=\"$1\" href=\"$_\">";
+        }
+    }
+
     my $title = $self->_template_text('pagetitle1', %hash);
     if($self->{cfg}->{colorscheme} ne "none") {
         print OUTPUT <<HTML;
@@ -325,7 +348,10 @@ HTML
 sub _pageheader
 {
     my $self = shift;
-    open(PAGEHEAD, $self->{cfg}->{pagehead}) or die("$0: Unable to open $self->{cfg}->{pagehead} for reading: $!\n");
+    unless(open(PAGEHEAD, $self->{cfg}->{pagehead})) {
+        warn("$0: Unable to open $self->{cfg}->{pagehead} for reading: $!\n");
+        return;
+    }
     while (<PAGEHEAD>) {
         _html($_);
     }
@@ -335,7 +361,10 @@ sub _pageheader
 sub _pagefooter
 {
     my $self = shift;
-    open(PAGEFOOT, $self->{cfg}->{pagefoot}) or die("$0: Unable to open $self->{cfg}->{pagefoot} for reading: $!\n");
+    unless(open(PAGEFOOT, $self->{cfg}->{pagefoot})) {
+        warn("$0: Unable to open $self->{cfg}->{pagefoot} for reading: $!\n");
+        return;
+    }
     while (<PAGEFOOT>) {
         _html($_);
     }
@@ -344,25 +373,21 @@ sub _pagefooter
 
 sub _activedays
 {
-    # The most actives times on the channel
+    # The most actives days on the channel
     my $self = shift;
     my $days = $self->{stats}->{days};
     my $ndays = $self->{cfg}->{dailyactivity};
-    my $image;
-    my $time;
-    my $day;
-
-    my (%output, $class);
 
     my $highest_value = 1;
-    for ($day = $days; $day > $days - $ndays ; $day--) {
-        if (defined($self->{stats}->{day_lines}{$day})) {
-            if ($self->{stats}->{day_lines}{$day} > $highest_value) {
-                $highest_value=$self->{stats}->{day_lines}{$day};
+    for (my $day = $days; $day > $days - $ndays ; $day--) {
+        if (defined($self->{stats}->{day_lines}->[$day])) {
+            if ($self->{stats}->{day_lines}->[$day] > $highest_value) {
+                $highest_value = $self->{stats}->{day_lines}->[$day];
             }
         } else {
             #there are only $days - $day days :)
             $ndays = $days - $day;
+            last;
         }
     }
 
@@ -373,22 +398,16 @@ sub _activedays
 
     _html("<table border=\"0\"><tr>\n");
 
-    for ($day = $days - $ndays +1; $day <= $days ; $day++) {
-        my $lines = $self->{stats}->{day_lines}{$day};
+    for (my $day = $days - $ndays + 1; $day <= $days ; $day++) {
+        my $lines = $self->{stats}->{day_lines}[$day];
         _html("<td align=\"center\" valign=\"bottom\" class=\"asmall\">$lines<br />");
-        for ($time = 4; $time >= 0; $time--) {
-            if (defined($self->{stats}->{day_times}{$day}[$time])) {
-                my $size = int(($self->{stats}->{day_times}{$day}[$time] / $highest_value) * 100);
+        for (my $time = 4; $time >= 0; $time--) {
+            if (defined($self->{stats}->{day_times}[$day][$time])) {
+                my $size = int(($self->{stats}->{day_times}[$day][$time] / $highest_value) * 100);
 
-                if ($size < 1) {
-                    # Opera doesn't understand '0.xxxx' in the height="xx" attr,
-                    # so we simply round up to 1 here.
-                    $size = 1;
-                }
-
-                $image = "pic_v_".$time*6;
+                my $image = "pic_v_".$time*6;
                 $image = $self->{cfg}->{$image};
-                _html("<img src=\"$self->{cfg}->{piclocation}/$image\" width=\"15\" height=\"$size\" alt=\"$lines\" /><br />");
+                _html("<img src=\"$self->{cfg}->{piclocation}/$image\" width=\"15\" height=\"$size\" alt=\"$lines\" title=\"$lines\" /><br />") if $size;
 
             }
         }
@@ -397,10 +416,9 @@ sub _activedays
 
     _html("</tr><tr>");
 
-    for ($b = $ndays-1; $b >= 0 ; $b--) {
-            $class = 'rankc10center';
-        _html("<td class=\"$class\" align=\"center\">$b</td>");
-}
+    for (my $day = $ndays - 1; $day >= 0 ; $day--) {
+        _html("<td class=\"rankc10center\" align=\"center\">$day</td>");
+    }
 
     _html("</tr></table>");
 
@@ -414,7 +432,7 @@ sub _activetimes
     # The most actives times on the channel
     my $self = shift;
 
-    my (%output, $class);
+    my (%output);
 
     $self->_headline($self->_template_text('activetimestopic'));
 
@@ -422,27 +440,17 @@ sub _activetimes
 
     my $highest_value = $self->{stats}->{times}{$toptime[0]};
 
-    my @now = localtime($self->{cfg}->{timestamp});
-
-    my $image;
-
     for my $hour (sort keys %{ $self->{stats}->{times} }) {
 
         my $size = int(($self->{stats}->{times}{$hour} / $highest_value) * 100);
         my $percent = ($self->{stats}->{times}{$hour} / $self->{stats}->{parsedlines}) * 100;
-        my $lines_per_hour = $self->{stats}->{times}{$hour};
         $percent =~ s/(\.\d)\d+/$1/;
+        my $lines_per_hour = $self->{stats}->{times}{$hour};
 
-        if ($size < 1) {
-            # Opera doesn't understand '0.xxxx' in the height="xx" attr,
-            # so we simply round up to 1 here.
-            $size = 1;
-        }
-
-        $image = "pic_v_".(int($hour/6)*6);
+        my $image = "pic_v_".(int($hour/6)*6);
         $image = $self->{cfg}->{$image};
 
-        $output{$hour} = "<td align=\"center\" valign=\"bottom\" class=\"asmall\">$percent%<br /><img src=\"$self->{cfg}->{piclocation}/$image\" width=\"15\" height=\"$size\" alt=\"$lines_per_hour\" title=\"$lines_per_hour\"/></td>\n";
+        $output{$hour} = "<td align=\"center\" valign=\"bottom\" class=\"asmall\">$percent%<br /><img src=\"$self->{cfg}->{piclocation}/$image\" width=\"15\" height=\"$size\" alt=\"$lines_per_hour\" title=\"$lines_per_hour\"/></td>\n" if $size;
     }
 
     _html("<table border=\"0\"><tr>\n");
@@ -450,7 +458,7 @@ sub _activetimes
     for ($b = 0; $b < 24; $b++) {
         $a = sprintf("%02d", $b);
 
-        if (!defined($output{$a}) || $output{$a} eq "") {
+        if (!defined($output{$a})) {
             _html("<td align=\"center\" valign=\"bottom\" class=\"asmall\">0%</td>");
         } else {
             _html($output{$a});
@@ -460,17 +468,13 @@ sub _activetimes
     _html("</tr><tr>");
 
     # Remove leading zero
-    $toptime[0] =~ s/0(\d)/$1/;
+    $toptime[0] =~ s/^0//;
 
     for ($b = 0; $b < 24; $b++) {
-        if ($toptime[0] == $b) {
-            # Highlight the top time
-            $class = 'hirankc10center';
-        } else {
-            $class = 'rankc10center';
-        }
+        # Highlight the top time
+        my $class = $toptime[0] == $b ? 'hirankc10center' : 'rankc10center';
         _html("<td class=\"$class\" align=\"center\">$b</td>");
-}
+    }
 
     _html("</tr></table>");
 
@@ -489,7 +493,7 @@ sub _activenicks
     _html("<table border=\"0\" width=\"$self->{cfg}->{tablewidth}\"><tr>");
     _html("<td>&nbsp;</td>"
     . "<td class=\"tdtop\"><b>" . $self->_template_text('nick') . "</b></td>"
-    . "<td class=\"tdtop\"><b>" . $self->_template_text('numberlines') . "</b></td>"
+    . ($self->{cfg}->{showlines} ? "<td class=\"tdtop\"><b>" . $self->_template_text('numberlines') . "</b></td>" : "")
     . ($self->{cfg}->{showtime} ? "<td class=\"tdtop\"><b>".$self->_template_text('show_time')."</b></td>" : "")
     . ($self->{cfg}->{showwords} ? "<td class=\"tdtop\"><b>".$self->_template_text('show_words')."</b></td>" : "")
     . ($self->{cfg}->{showwpl} ? "<td class=\"tdtop\"><b>".$self->_template_text('show_wpl')."</b></td>" : "")
@@ -513,7 +517,7 @@ sub _activenicks
     for (my $c = 0; $c < $self->{cfg}->{activenicks}; $c++) {
         last unless $self->{cfg}->{userpics};
         my $nick = $active[$c];
-        if ($self->{users}->{userpics}{$nick}) {
+        if ($self->{users}->{userpics}{$nick} or $self->{cfg}->{defaultpic}) {
             _html("<td class=\"tdtop\"" .
                 ($self->{cfg}->{userpics} > 1 ? " colspan=\"$self->{cfg}->{userpics}\"" : "") .
                 "><b>" . $self->_template_text('userpic') ."</b></td>");
@@ -559,12 +563,13 @@ sub _activenicks
 
         if ($self->{cfg}->{showlastseen}) {
             $lastseen = $self->{stats}->{days} - $self->{stats}->{lastvisited}{$nick};
+            my %hash = ( days => $lastseen );
             if ($lastseen == 0) {
                 $lastseen = $self->_template_text('today');
             } elsif ($lastseen == 1) {
-                $lastseen = "$lastseen " .$self->_template_text('lastseen1');
+                $lastseen = $self->_template_text('lastseen1', %hash);
             } else {
-                $lastseen = "$lastseen " .$self->_template_text('lastseen2');
+                $lastseen = $self->_template_text('lastseen2', %hash);
             }
         }
 
@@ -578,9 +583,10 @@ sub _activenicks
        . ($sex ? ($sex eq 'm' ? " class=\"male\">"
        : ($sex eq 'f' ? " class=\"female\">" : " class=\"bot\">")) : ">")
        ."$visiblenick</td>"
-        . ($self->{cfg}->{showlinetime} ?
+        . ($self->{cfg}->{showlines} ? 
+         ($self->{cfg}->{showlinetime} ?
         "<td style=\"background-color: $color\" nowrap=\"nowrap\">".$self->_user_linetimes($nick,$active[0])."</td>"
-        : "<td style=\"background-color: $color\">$line</td>")
+        : "<td style=\"background-color: $color\">$line</td>") : "")
         . ($self->{cfg}->{showtime} ?
         "<td style=\"background-color: $color\">".$self->_user_times($nick)."</td>"
         : "")
@@ -627,8 +633,15 @@ sub _activenicks
         if ($self->{cfg}->{activenicks} <  $remain) {
             _html("<br /><b><i>" . $self->_template_text('nottop') . "</i></b><table><tr>");
             for (my $i = $self->{cfg}->{activenicks}; $i < $remain; $i++) {
+                my $visiblenick;
+                my $nick = $active[$i];
                 unless ($i % 5) { if ($i != $self->{cfg}->{activenicks}) { _html("</tr><tr>"); } }
                 my $items;
+                if ($self->{users}->{userlinks}{$nick}) {
+                    $visiblenick = $self->_format_word($self->{users}->{userlinks}{$nick}, $nick);
+                } else {
+                    $visiblenick = $self->_format_word($nick);
+                }
                 if ($self->{cfg}->{sortbywords}) {
                     $items = $self->{stats}->{words}{$active[$i]};
                 } else {
@@ -638,7 +651,7 @@ sub _activenicks
                 _html("<td class=\"rankc10\">"
                 . ($sex ? ($sex eq 'm' ? "<span class=\"male\">"
                 : ($sex eq 'f' ? "<span class=\"female\">" : "<span class=\"bot\">")) : "")
-                ."$active[$i] ($items)"
+                ."$visiblenick ($items)"
                 . ($sex ? "</span>" : "")
                 ."</td>");
             }
@@ -1549,6 +1562,15 @@ sub _lasttopics
 
             $hash{nick} = $nick;
             $hash{time} = "$hour:$min";
+            $hash{days} = $self->{stats}->{days} - $self->{stats}->{topics}[$i]{days};
+            if ($hash{days} == 0) {
+                $hash{date} = $self->_template_text('today');
+            } elsif ($hash{days} == 1) {
+                $hash{date} = $self->_template_text('lastseen1', %hash);
+            } else {
+                $hash{date} = $self->_template_text('lastseen2', %hash);
+            }
+
             _html('<tr><td class="hicell"><i>' . $self->_format_line($topic) . '</i></td>');
             _html('<td class="hicell"><b>' . $self->_template_text('bylinetopic', %hash) . '</b></td></tr>');
         }
@@ -1569,14 +1591,14 @@ sub _template_text
 
     my $text;
 
-    unless ($text = $self->{tmps}->{lc($self->{cfg}->{lang})}{$template}) {
+    unless ($text = $self->{tmps}->{$self->{cfg}->{lang}}{$template}) {
         # Fall back to English if the language template doesn't exist
 
-        if ($text = $self->{tmps}->{en}{$template}) {
-            print "Note: No translation in '$self->{cfg}->{lang}' for '$template' - falling back to English..\n"
+        if ($text = $self->{tmps}->{EN}{$template}) {
+            print "Note: No translation in '$self->{cfg}->{lang}' for '$template' - falling back to English.\n"
                 unless ($self->{cfg}->{silent});
         } else {
-            die("No such template '$template' in language file.\n");
+            die("No template for '$template' in language file.\n");
         }
     }
     if($self->{iconv}) {
@@ -1697,7 +1719,7 @@ sub _mostusedword
             next if is_ignored($popular[$i]);
 
             my $a = $count + 1;
-            my $popular = $self->_format_word($popular[$i]);
+            my $popular = $self->_format_word($self->{stats}->{word_upcase}{$popular[$i]});
             my $wordcount = $self->{stats}->{wordcounts}{$popular[$i]};
             my $lastused = $self->_format_word($self->{stats}->{wordnicks}{$popular[$i]});
             my $class;
@@ -1781,11 +1803,14 @@ sub _mostreferencednicks
         _html("<td class=\"tdtop\"><b>" . $self->_template_text('lastused') . "</b></td></tr>");
 
         for(my $i = 0; $i < $self->{cfg}->{nickhistory}; $i++) {
-            last unless $i <= $#popular;
+            last if $i >= @popular;
             my $a = $i + 1;
-            my $popular   = $self->_format_word($popular[$i]);
+            my $popular   = is_nick($popular[$i]) || $self->{stats}->{word_upcase}{$popular[$i]} || $popular[$i];
+            $popular      = $self->_format_word($popular[$i]);
             my $wordcount = $self->{stats}->{wordcounts}{$popular[$i]};
-            my $lastused  = $self->_format_word($self->{stats}->{wordnicks}{$popular[$i]});
+            my $lastused  = $self->_format_word($self->{stats}->{wordnicks}{$popular[$i]} || "");
+            # this is undefined when a nick is referenced before being used
+            # first (this is a minor bug we ignore here, see test/01.cfg)
 
             my $class;
             if ($a == 1) {
@@ -1803,6 +1828,90 @@ sub _mostreferencednicks
     }
 }
 
+sub _karma
+{
+    # List showing the most referenced nicks
+    my $self = shift;
+
+    my %karma;
+
+    foreach my $thing (sort keys %{ $self->{stats}->{karma} }) {
+        my $Thing = lc(is_nick($thing) || $thing); # FIXME: this is ugly
+        foreach my $nick (keys %{ $self->{stats}->{karma}{$thing} }) {
+            $karma{$Thing} += $self->{stats}->{karma}{$thing}{$nick};
+        }
+    }
+
+    my @popular = sort { $karma{$b} <=> $karma{$a} } keys %karma;
+    return unless @popular;
+
+    $self->_headline($self->_template_text('karmatopic'));
+
+    _html("<table border=\"0\" width=\"$self->{cfg}->{tablewidth}\"><tr>");
+    _html("<td>&nbsp;</td><td class=\"tdtop\"><b>" . $self->_template_text('nick') . "</b></td>");
+    _html("<td class=\"tdtop\"><b>" . $self->_template_text('karma') . "</b></td>");
+    _html("<td class=\"tdtop\"><b>" . $self->_template_text('goodkarma') . "</b></td>");
+    _html("<td class=\"tdtop\"><b>" . $self->_template_text('badkarma') . "</b></td></tr>");
+
+    my @goodpos = grep { $karma{$_} > 0 } @popular;
+    splice @goodpos, $self->{cfg}->{karmahistory}, @goodpos
+        if @goodpos > $self->{cfg}->{karmahistory};
+    my @badpos = grep { $karma{$_} < 0 } @popular;
+    splice @badpos, 0, @badpos - $self->{cfg}->{karmahistory}
+        if @badpos > $self->{cfg}->{karmahistory};
+
+    my $pos = 0;
+    foreach my $thing (@goodpos) {
+        my $class = ($pos++ == 0) ? 'hirankc' : 'rankc';
+        my $Thing = $self->_format_word(is_nick($thing) || $thing); # ugliness #2
+        _html("<tr><td class=\"$class\">$pos</td>");
+        _html("<td class=\"hicell\">$Thing</td>");
+        _html("<td class=\"hicell\">$karma{$thing}</td>");
+
+        my @k = grep { $self->{stats}->{karma}{$thing}{$_} > 0 }
+            (keys %{ $self->{stats}->{karma}{$thing} });
+        @k = $self->_trimlist(@k);
+        my $n = join ', ', map { $self->_format_word($_) } @k;
+        _html("<td class=\"hicell\">$n</td>");
+
+        @k = grep { $self->{stats}->{karma}{$thing}{$_} < 0 }
+            (keys %{ $self->{stats}->{karma}{$thing} });
+        @k = $self->_trimlist(@k);
+        $n = join ', ', map { $self->_format_word($_) } @k;
+        _html("<td class=\"hicell\">$n</td>");
+        _html("</tr>");
+    }
+
+    if (@goodpos and @badpos) {
+            _html("<tr><td class=\"rankc\"></td>");
+            _html("<td class=\"hicell\" colspan=\"4\"></td>");
+        _html("</tr>");
+    }
+
+    $pos = @badpos;
+    foreach my $thing (@badpos) {
+        my $class = ($pos == 1) ? 'hirankc' : 'rankc';
+        my $Thing = $self->_format_word(is_nick($thing) || $thing);
+        _html("<tr><td class=\"$class\">". ($pos--) ."</td>");
+        _html("<td class=\"hicell\">$Thing</td>");
+        _html("<td class=\"hicell\">$karma{$thing}</td>");
+
+        my @k = grep { $self->{stats}->{karma}{$thing}{$_} > 0 }
+            (keys %{ $self->{stats}->{karma}{$thing} });
+        @k = $self->_trimlist(@k);
+        my $n = join ', ', map { $self->_format_word($_) } @k;
+        _html("<td class=\"hicell\">$n</td>");
+
+        @k = grep { $self->{stats}->{karma}{$thing}{$_} < 0 }
+            (keys %{ $self->{stats}->{karma}{$thing} });
+        @k = $self->_trimlist(@k);
+        $n = join ', ', map { $self->_format_word($_) } @k;
+        _html("<td class=\"hicell\">$n</td>");
+        _html("</tr>");
+    }
+    _html("</table>");
+}
+
 sub _mosturls
 {
     # List showing the most referenced URLs
@@ -1831,12 +1940,7 @@ sub _mosturls
             }
             $printurl = htmlentities($printurl, $self->{cfg}->{charset});
             my $linkurl = urlencode($sorturls[$i]);
-            my $class;
-            if ($a == 1) {
-                $class = 'hirankc';
-            } else {
-                $class = 'rankc';
-            }
+            my $class = ($a == 1) ? 'hirankc' : 'rankc';
             _html("<tr><td class=\"$class\">$a</td>");
             _html("<td class=\"hicell\"><a href=\"$linkurl\">$printurl</a></td>");
             _html("<td class=\"hicell\">$urlcount</td>");
@@ -1847,6 +1951,43 @@ sub _mosturls
     }
 }
 
+sub _charts
+{
+    # List showing the most played songs
+    my $self = shift;
+
+    my @sortcharts = sort { $self->{stats}->{chartcounts}{$b} <=> $self->{stats}->{chartcounts}{$a} }
+                        keys %{ $self->{stats}->{chartcounts} };
+
+    if (@sortcharts) {
+
+        $self->_headline($self->_template_text('chartstopic'));
+
+        _html("<table border=\"0\" width=\"$self->{cfg}->{tablewidth}\"><tr>");
+        _html("<td>&nbsp;</td><td class=\"tdtop\"><b>" . $self->_template_text('song') . "</b></td>");
+        _html("<td class=\"tdtop\"><b>" . $self->_template_text('numberplayed') . "</b></td>");
+        _html("<td class=\"tdtop\"><b>" . $self->_template_text('playedby') . "</b></td></tr>");
+
+        for(my $i = 0; $i < $self->{cfg}->{chartshistory}; $i++) {
+            last unless $i < @sortcharts;
+            my $a = $i + 1;
+            my $song = $sortcharts[$i];
+            my $chartcount = $self->{stats}->{chartcounts}{$song};
+            my $lastused = $self->{stats}->{chartnicks}{$song};
+            $song = $self->{stats}->{word_upcase}{$song};
+            $song = substr($song, 0, 60) if (length($song) > 60);
+            $song = $self->_format_word($song);
+            my $class = ($a == 1) ? 'hirankc' : 'rankc';
+            _html("<tr><td class=\"$class\">$a</td>");
+            _html("<td class=\"hicell\">$song</td>");
+            _html("<td class=\"hicell\">$chartcount</td>");
+            _html("<td class=\"hicell\">$lastused</td>");
+            _html("</tr>");
+        }
+        _html("</table>");
+    }
+}
+
 sub _legend
 {
     # A legend showing the timebars and their associated time.
@@ -1917,11 +2058,12 @@ sub _user_linetimes
     my $len      = ($self->{stats}->{lines}{$nick} / $self->{stats}->{lines}{$top}) * 100;
 
     for (my $i = 0; $i <= 3; $i++) {
-        next if not defined $self->{stats}->{line_times}{$nick}[$i];
-        my $w = int(($self->{stats}->{line_times}{$nick}[$i] / $self->{stats}->{lines}{$nick}) * $len);
+        my $l = $self->{stats}->{line_times}{$nick}[$i];
+        next if not defined $l;
+        my $w = int(($l / $self->{stats}->{lines}{$nick}) * $len);
         if ($w) {
             my $pic = 'pic_h_'.(6*$i);
-            $bar .= "<img src=\"$self->{cfg}->{piclocation}/$self->{cfg}->{$pic}\" border=\"0\" width=\"$w\" height=\"15\" align=\"middle\" alt=\"$self->{stats}->{line_times}{$nick}[$i]\" />";
+            $bar .= "<img src=\"$self->{cfg}->{piclocation}/$self->{cfg}->{$pic}\" border=\"0\" width=\"$w\" height=\"15\" align=\"middle\" alt=\"$l\" title=\"$l\" />";
         }
     }
     return "$bar&nbsp;$self->{stats}->{lines}{$nick}";
@@ -1959,11 +2101,12 @@ sub _user_times
     my $timestat = ($self->{cfg}->{sortbywords} ? 'word_times' : 'line_times');
     
     for (my $i = 0; $i <= 3; $i++) {
-        next if not defined $self->{stats}->{$timestat}{$nick}[$i];
-        my $w = int(($self->{stats}->{$timestat}{$nick}[$i] / $self->{stats}->{$itemstat}{$nick}) * 40);
+        my $l = $self->{stats}->{$timestat}{$nick}[$i];
+        next if not defined $l;
+        my $w = int(($l / $self->{stats}->{$itemstat}{$nick}) * 40);
         if ($w) {
             my $pic = 'pic_h_'.(6*$i);
-            $bar .= "<img src=\"$self->{cfg}->{piclocation}/$self->{cfg}->{$pic}\" border=\"0\" width=\"$w\" height=\"15\" alt=\"\" />";
+            $bar .= "<img src=\"$self->{cfg}->{piclocation}/$self->{cfg}->{$pic}\" border=\"0\" width=\"$w\" height=\"15\" alt=\"$l\" title=\"$l\" />";
         }
     }
     return $bar;
@@ -1983,13 +2126,14 @@ sub _user_pic
     my $biguserpic = $self->{users}->{biguserpics}{$nick};
     if ($biguserpic) {
         $biguserpic = $self->{cfg}->{imagepath} .
-            randomglob($biguserpic, $self->{cfg}->{imageglobpath})
+            randomglob($biguserpic, $self->{cfg}->{imageglobpath}, $nick)
                 if $biguserpic !~ /^http:\/\//i;
         $output .= "<a href=\"$biguserpic\">";
     }
 
     my $pic = $self->{users}->{userpics}{$nick} || $self->{cfg}->{defaultpic};
-    $pic = $self->{cfg}->{imagepath} . randomglob($pic, $self->{cfg}->{imageglobpath})
+    $pic = $self->{cfg}->{imagepath} .
+        randomglob($pic, $self->{cfg}->{imageglobpath}, $nick)
         unless $pic =~ /^http:\/\//i;
     my $height = $self->{cfg}->{picheight} ? " height=\"$self->{cfg}->{picheight}\"" : "";
     my $width = $self->{cfg}->{picwidth} ? " width=\"$self->{cfg}->{picwidth}\"" : "";
@@ -2025,7 +2169,8 @@ sub _mostnicks
         foreach my $nick (@sortnicks) {
             next if is_ignored($nick);
             my $nickcount = keys %{ $self->{stats}->{nicks}->{$nick} };
-            my $nickused = join(", ", values %{ $self->{stats}->{nicks}->{$nick} });
+            my @nicks = values %{ $self->{stats}->{nicks}->{$nick} };
+            my $nickused = join(", ", $self->_trimlist(@nicks));
 
             last unless ($nickcount > 1);
 
@@ -2185,6 +2330,62 @@ sub _istoponly {
     }
 }
 
+sub _trimlist {
+    my $self = shift;
+    return @_ unless $self->{cfg}->{nicklimit};
+    splice @_, $self->{cfg}->{nicklimit}, @_, qw/.../ if @_ > $self->{cfg}->{nicklimit};
+    return @_;
+}
+
+sub _activegenders {
+    # The most active gender in the channel
+    my $self = shift;
+    my @topgender = sort {$self->{stats}->{sex_lines}{$b} <=> $self->{stats}->{sex_lines}{$a}} keys %{$self->{stats}->{sex_lines}};
+
+    return unless @topgender;
+
+    $self->_headline($self->_template_text('activegenderstopic'));
+    _html("<table border=\"0\" width=\"$self->{cfg}->{tablewidth}\"><tr>");
+    _html(" <td>&nbsp;</td>"
+    . "<td class=\"tdtop\"><b>" . $self->_template_text('gender') . "</b></td>"
+    . "<td class=\"tdtop\"><b>" . $self->_template_text('numberlines') . "</b></td>"
+    . "<td class=\"tdtop\"><b>" . $self->_template_text('nick') . "</b></td>"
+    );
+
+    my $i = 1;
+    for my $gender (@topgender) {
+        my $bar = "";
+        my $len = ($self->{stats}->{sex_lines}{$gender} / $self->{stats}->{sex_lines}{$topgender[0]}) * 100;
+
+        for (0 .. 3) {
+            next if not defined $self->{stats}->{sex_line_times}{$gender}[$_];
+            my $w = int(($self->{stats}->{sex_line_times}{$gender}[$_] / $self->{stats}->{sex_lines}{$gender}) * $len);
+            if ($w) {
+                my $pic = 'pic_h_'.(6*$_);
+                $bar .= "<img src=\"$self->{cfg}->{piclocation}/$self->{cfg}->{$pic}\" border=\"0\" width=\"$w\" height=\"15\" align=\"middle\" alt=\"$self->{stats}->{sex_line_times}{$gender}[$_]\" />";
+            }
+        }
+
+        my @top_active = sort { $self->{stats}->{lines}{$b} <=> $self->{stats}->{lines}{$a} }
+            grep { $self->{users}->{sex}{$_} and $self->{users}->{sex}{$_} eq $gender }
+            keys %{ $self->{stats}->{lines} };
+        my $nicklist = join ", ",
+            map { $_ . ($self->{stats}->{lines}{$_} ? " ($self->{stats}->{lines}{$_})" : "") }
+            $self->_trimlist(@top_active);
+
+        my $class = ($i == 1 ? "hirankc" : "rankc");
+        my $span_class = $gender eq 'm' ? "male" : ($gender eq 'f' ? "female" : "bot");
+        _html("</tr><tr>");
+        _html(" <td class=\"$class\" align=\"left\">$i</td>");
+        _html(" <td class=\"hicell\"><span class=\"$span_class\">" . $self->_template_text("gender_$gender") . "</span></td>");
+        _html(" <td class=\"hicell\"><span style=\"white-space:nowrap;\">$bar</span> $self->{stats}->{sex_lines}{$gender}</td>");
+        _html(" <td class=\"hicell\">$nicklist</td>");
+        $i++;
+    }
+
+    _html("</tr></table>");
+}
+
 1;
 
 __END__
@@ -2234,7 +2435,8 @@ Morten Brix Pedersen <morten@wtf.dk>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2001-2002 Morten Brix Pedersen. All rights reserved.
+Copyright (C) 2001-2005 Morten Brix Pedersen. All rights reserved.
+Copyright (C) 2003-2005 Christoph Berg <cb@df7cb.de>.
 This program is free software; you can redistribute it and/or modify it
 under the terms of the GPL, license is included with the distribution of
 this file.
