@@ -6,6 +6,11 @@ package Pisg::Parser::Logfile;
 use strict;
 $^W = 1;
 
+# the log cache
+use Data::Dumper;
+$Data::Dumper::Indent = 1;
+my $cache;
+
 # test for Text::Iconv
 my $have_iconv = 1;
 eval 'use Text::Iconv';
@@ -14,11 +19,7 @@ $have_iconv = 0 if $@;
 sub new
 {
     my $type = shift;
-    my %args = @_;
-    my $self = {
-        cfg => $args{cfg},
-        parser => undef
-    };
+    my $self = shift; # get cfg and users
 
     # Import common functions in Pisg::Common
     require Pisg::Common;
@@ -53,9 +54,10 @@ sub new
     }
 
     # precompile the regexps used (we can't use /o since the config might be different per channel)
-    $self->{foulwords_regexp} = qr/($self->{cfg}->{foulwords})/i;
-    $self->{ignorewords_regexp} = qr/$self->{cfg}->{ignorewords}/i;
-    $self->{violentwords_regexp} = qr/^($self->{cfg}->{violentwords}) (\S+)(.*)/i;
+    $self->{foulwords_regexp} = qr/($self->{cfg}->{foulwords})/i if $self->{cfg}->{foulwords};
+    $self->{ignorewords_regexp} = qr/$self->{cfg}->{ignorewords}/i if $self->{cfg}->{ignorewords};
+    $self->{violentwords_regexp} = qr/^($self->{cfg}->{violentwords}) (\S+)(.*)/i if $self->{cfg}->{violentwords};
+    $self->{chartsregexp} = qr/^$self->{cfg}->{chartsregexp}/i if $self->{cfg}->{chartsregexp};
 
     return $self;
 }
@@ -83,74 +85,105 @@ _END
 sub analyze
 {
     my $self = shift;
-    my (%stats, %lines);
 
-    if (defined $self->{parser}) {
-        my $starttime = time();
-
-        # Just initialize these to 0
-        $stats{days} = 0;
-        $stats{parsedlines} = 0;
-        $stats{totallines} = 0;
-
-        if (scalar(@{$self->{cfg}->{logdir}}) > 0) {
-            # Run through all files in dir
-            $self->_parse_dir(\%stats, \%lines);
-        } else {
-            my %state = (
-                linecount  => 0,
-                lastnick   => '',
-                monocount  => 0,
-                lastnormal => '',
-                oldtime    => 24
-            );
-            foreach my $logfile (@{$self->{cfg}->{logfile}}) {
-                # Run through the whole logfile
-                $self->_parse_file(\%stats, \%lines, $logfile, \%state);
-            }
-        }
-
-        $self->_pick_random_lines(\%stats, \%lines);
-        _uniquify_nicks(\%stats);
-
-        my ($sec,$min,$hour) = gmtime(time() - $starttime);
-        my $processtime = sprintf('%02d hours, %02d minutes and %02d seconds', $hour, $min, $sec);
-
-        $stats{processtime}{hours} = sprintf('%02d', $hour);
-        $stats{processtime}{mins} = sprintf('%02d', $min);
-        $stats{processtime}{secs} = sprintf('%02d', $sec);
-
-        print "Channel analyzed successfully in $processtime on ",
-        scalar localtime(time()), "\n"
-            unless ($self->{cfg}->{silent});
-
-        return \%stats;
-
-    } else {
+    unless (defined $self->{parser}) {
         print STDERR "Skipping channel '$self->{cfg}->{channel}' due to lack of parser.\n";
         return undef
     }
 
-    # Shouldn't get here.
-    return undef;
+    my $starttime = time();
+
+    my @logfiles = @{$self->{cfg}->{logfile}};
+    # expand wildcards
+    @logfiles = map { if(/[\[*?]/) { glob; } else { $_; } } @logfiles;
+
+    foreach my $logdir (@{$self->{cfg}->{logdir}}) {
+        push @logfiles, $self->_parse_dir($logdir); # get all files in dir
+    }
+
+    my $count = @logfiles;
+    my $shift = 0;
+    if($self->{cfg}->{nfiles} > 0) { # chop list to maximal length
+        $shift = @logfiles - $self->{cfg}->{nfiles};
+        splice(@logfiles, 0, $shift) if $shift > 0;
+    }
+
+    unless ($self->{cfg}->{silent}) {
+        my $msg = "";
+        $msg = ", parsing the last $self->{cfg}->{nfiles}" if ($shift > 0);
+        print "$count logfile(s) found$msg, using $self->{cfg}->{format} format...\n\n"
+    }
+
+    my (%stats, %lines);
+    %stats = (
+        oldtime => 24,
+        days => 0,
+        lastnick   => '',
+        monocount  => 0,
+        day_lines => [ undef ],
+        day_times => [ undef ],
+    );
+
+    foreach my $logfile (@logfiles) {
+        # Run through the logfile
+        print "Analyzing log $logfile... " unless ($self->{cfg}->{silent});
+
+        my $s = {
+            oldtime => 24,
+            days => 0,
+            firsttime => 0,
+            lastnick => '',
+            parsedlines => 0,
+            totallines => 0,
+        };
+        my $l = {};
+
+        unless ($self->{cfg}->{cachedir} and $self->_read_cache(\$s, \$l, $logfile)) {
+            $self->_parse_file($s, $l, $logfile);
+            if ($self->{cfg}->{cachedir}) {
+                $self->_update_cache($s, $l, $logfile);
+            }
+        }
+        $self->_merge_stats(\%stats, $s); # merge per-file stats into global stats
+        $self->_merge_lines(\%lines, $l);
+
+        print "$stats{days} days, $stats{parsedlines} lines total\n"
+            unless ($self->{cfg}->{silent});
+    }
+
+    if ($self->{cfg}->{statsdump}) {
+        open C, "> $self->{cfg}->{statsdump}" or die "$self->{cfg}->{statsdump}: $!";
+        print C Data::Dumper->Dump([\%stats, \%lines], ["stats", "lines"]);
+        close C;
+    }
+
+    $self->_pick_random_lines(\%stats, \%lines);
+    _uniquify_nicks(\%stats);
+
+    my ($sec,$min,$hour) = gmtime(time() - $starttime);
+    my $processtime = sprintf('%02d hours, %02d minutes and %02d seconds', $hour, $min, $sec);
+
+    $stats{processtime}{hours} = sprintf('%02d', $hour);
+    $stats{processtime}{mins} = sprintf('%02d', $min);
+    $stats{processtime}{secs} = sprintf('%02d', $sec);
+
+    print "Channel analyzed successfully in $processtime on ",
+    scalar localtime(time()), "\n\n"
+        unless ($self->{cfg}->{silent});
+
+    return \%stats;
 }
 
 sub _parse_dir
 {
     my $self = shift;
-    my ($stats, $lines) = @_;
+    my $logdir = shift;
 
-    # Loop through each logdir we were given
-    foreach my $logdir (@{$self->{cfg}->{logdir}}) {
         # Add trailing slash when it's not there..
         $logdir =~ s/([^\/])$/$1\//;
 
         unless ($self->{cfg}->{silent}) {
-            unless ($self->{cfg}->{nfiles} > 0) {
-                print "Going into $logdir and parsing all files there...\n\n"
-            } else {
-                print "Going into $logdir and parsing the last $self->{cfg}->{nfiles} file(s) there...\n\n"
-            }
+            print "Looking for logfiles in $logdir...\n\n"
         }
         my @filesarray;
         opendir(LOGDIR, $logdir) or
@@ -161,11 +194,6 @@ sub _parse_dir
         die("No files in \"$logdir\" matched prefix \"$self->{cfg}->{logprefix}\"");
         closedir(LOGDIR);
 
-        my %state = (
-            lastnick   => '',
-            monocount  => 0,
-            oldtime    => 24
-        );
         if ($self->{cfg}->{logsuffix} ne '') {
             my @temparray;
             my %months = (
@@ -224,38 +252,25 @@ sub _parse_dir
                     }
                 }
             }
-            my @newarray = @filesarray[ sort {
+            @filesarray = @filesarray[ sort {
                                         $year[$a] <=> $year[$b]
                                                 ||
                                         $month[$a] <=> $month[$b]
                                                 ||
                                         $day[$a] <=> $day[$b]
                                     } 0..$#filesarray ];
-            @filesarray = @newarray;
         } else {
             @filesarray = sort {lc($a) cmp lc($b)} @filesarray;
         }
 
-        if($self->{cfg}->{nfiles} > 0) {
-            my $shift = @filesarray - $self->{cfg}->{nfiles};
-            splice(@filesarray, 0, $shift) if $shift > 0;
-        }
-
-        foreach my $file (@filesarray) {
-            $file = $logdir . $file;
-            $self->_parse_file($stats, $lines, $file, \%state);
-        }
-    }
+        return map { "$logdir$_" } @filesarray;
 }
 
 # This parses the file...
 sub _parse_file
 {
     my $self = shift;
-    my ($stats, $lines, $file, $state) = @_;
-
-    print "Analyzing log $file in '$self->{cfg}->{format}' format...\n"
-        unless ($self->{cfg}->{silent});
+    my ($stats, $lines, $file) = @_;
 
     if ($file =~ /.bz2?$/ && -f $file) {
         open (LOGFILE, "bunzip2 -c $file |") or
@@ -268,8 +283,6 @@ sub _parse_file
         die("$0: Unable to open logfile($file): $!\n");
     }
 
-    my $lastnormal = '';
-
     while(my $line = <LOGFILE>) {
         $line = _strip_mirccodes($line);
         $line =~ s/\r+$//;       # Strip DOS Formatting
@@ -302,7 +315,7 @@ sub _parse_file
             for ($i = 0; $i <= $repeated; $i++) {
 
                 if ($i > 0) {
-                    $hashref = $self->{parser}->normalline($lastnormal, $.);
+                    $hashref = $self->{parser}->normalline($stats->{lastnormal}, $.);
                     #Increment number of lines for repeated lines
                 }
 
@@ -311,39 +324,37 @@ sub _parse_file
                 checkname($hashref->{nick}, $nick, $stats) if ($self->{cfg}->{showmostnicks});
                 $saying = $hashref->{saying};
 
-                if ($hour < $state->{oldtime}) {
+                if ($hour < $stats->{oldtime}) {
+                    $stats->{firsttime} = $hour if $stats->{oldtime} == 24; # save stamp for merging
                     $stats->{days}++;
-                    $stats->{day_times}{$stats->{days}}[0] = 0;
-                    $stats->{day_times}{$stats->{days}}[1] = 0;
-                    $stats->{day_times}{$stats->{days}}[2] = 0;
-                    $stats->{day_times}{$stats->{days}}[3] = 0;
-                    $stats->{day_lines}{$stats->{days}} = 0;
+                    @{$stats->{day_times}[$stats->{days}]} = (0, 0, 0, 0);
+                    $stats->{day_lines}->[$stats->{days}] = 0;
                 }
-                $state->{oldtime} = $hour;
+                $stats->{oldtime} = $hour;
 
                 if (!is_ignored($nick)) {
                     $stats->{parsedlines}++;
 
                     # Timestamp collecting
                     $stats->{times}{$hour}++;
-                    $stats->{day_times}{$stats->{days}}[int($hour/6)]++;
-                    $stats->{day_lines}{$stats->{days}}++;
+                    $stats->{day_times}[$stats->{days}][int($hour/6)]++;
+                    $stats->{day_lines}->[$stats->{days}]++;
 
                     $stats->{lines}{$nick}++;
                     $stats->{lastvisited}{$nick} = $stats->{days};
                     $stats->{line_times}{$nick}[int($hour/6)]++;
 
                     # Count up monologues
-                    if ($state->{lastnick} eq $nick) {
-                        $state->{monocount}++;
+                    if ($stats->{lastnick} eq $nick) {
+                        $stats->{monocount}++;
 
-                        if ($state->{monocount} == 5) {
+                        if ($stats->{monocount} == 5) {
                             $stats->{monologues}{$nick}++;
                         }
                     } else {
-                        $state->{monocount} = 0;
+                        $stats->{monocount} = 0;
                     }
-                    $state->{lastnick} = $nick;
+                    $stats->{lastnick} = $nick;
 
                     my $len = length($saying);
                     if ($len > $self->{cfg}->{minquote} && $len < $self->{cfg}->{maxquote}) {
@@ -364,28 +375,37 @@ sub _parse_file
 
                     if ($saying !~ /[a-z]/o && $saying =~ /[A-Z]/o) {
                         # Ignore single smileys on a line. eg. '<user> :P'
-                        if ($saying !~ /^[8;:=][ ^-o]?[)pPD}\]>]$/o) {
+                        if ($saying !~ /^[8;:=][ ^-o]?[)pPD\}\]>]$/o) {
                             $stats->{allcaps}{$nick}++;
                             push @{ $lines->{allcaplines}{$nick} }, $line;
                         }
                     }
 
-                    if (my @foul = $saying =~ /($self->{foulwords_regexp})/) {
+                    if ($self->{foulwords_regexp} and my @foul = $saying =~ /$self->{foulwords_regexp}/) {
                         $stats->{foul}{$nick} += scalar @foul;
                         push @{ $lines->{foullines}{$nick} }, $line;
                     }
 
-
                     # Who smiles the most?
                     # A regex matching al lot of smilies
                     $stats->{smiles}{$nick}++
-                        if ($saying =~ /[8;:=][ ^-o]?[)pPD}\]>]/o);
+                        if ($saying =~ /[8;:=][ ^-o]?[)pPD\}\]>]/o);
 
-                    if ($saying =~ /[8;:=][ ^-]?[\(\[\\\/{]/o and
+                    if ($saying =~ /[8;:=][ ^-]?[\(\[\\\/\{]/o and
                         $saying !~ /\w+:\/\//o) {
                         $stats->{frowns}{$nick}++;
                     }
 
+                    if ($self->{cfg}->{showkarma}) {
+                        # require 2 chars (catches C++), nick must not end in [+=-]
+                        if ($saying =~ /^(\S*\w\S*\w\S*(?<![+=-]))(\+\+|==|--)$/o) {
+                            my $thing = lc $1;
+                            my $k = $2 eq "++" ? 1 : ($2 eq "==" ? 0 : -1);
+                            $stats->{karma}{$thing}{$nick} = $k
+                                if !is_ignored($thing) and $thing ne lc($nick);
+                        }
+                    }
+
                     # Find URLs
                     if (my @urls = match_urls($saying)) {
                         foreach my $url (@urls) {
@@ -396,12 +416,21 @@ sub _parse_file
                         }
                     }
 
+                    if ($saying =~ /$self->{chartsregexp}/i) {
+                        $self->_charts($1, $nick);
+                    }
+
+                    if (my $s = $self->{users}->{sex}{$nick}) {
+                        $stats->{sex_lines}{$s}++;
+                        $stats->{sex_line_times}{$s}[int($hour/6)]++;
+                    }
+
                     _parse_words($stats, $saying, $nick, $self->{ignorewords_regexp}, $hour);
-                }
-            }
-            $lastnormal = $line;
+                } # ignored
+            } # repeated
+            $stats->{lastnormal} = $line;
             $repeated = 0;
-        }
+        } # normal lines
 
         # Match action lines.
         elsif ($hashref = $self->{parser}->actionline($line, $.)) {
@@ -414,22 +443,20 @@ sub _parse_file
             checkname($hashref->{nick}, $nick, $stats) if ($self->{cfg}->{showmostnicks});
             $saying = $hashref->{saying};
 
-            if ($hour < $state->{oldtime}) {
+            if ($hour < $stats->{oldtime}) {
+                $stats->{firsttime} = $hour if $stats->{oldtime} == 24; # save stamp for merging
                 $stats->{days}++;
-                $stats->{day_times}{$stats->{days}}[0] = 0;
-                $stats->{day_times}{$stats->{days}}[1] = 0;
-                $stats->{day_times}{$stats->{days}}[2] = 0;
-                $stats->{day_times}{$stats->{days}}[3] = 0;
-                $stats->{day_lines}{$stats->{days}} = 0;
+                @{$stats->{day_times}[$stats->{days}]} = (0, 0, 0, 0);
+                $stats->{day_lines}->[$stats->{days}] = 0;
             }
 
-            $state->{oldtime} = $hour;
+            $stats->{oldtime} = $hour;
 
             if (!is_ignored($nick)) {
                 # Timestamp collecting
                 $stats->{times}{$hour}++;
-                $stats->{day_times}{$stats->{days}}[int($hour/6)]++;
-                $stats->{day_lines}{$stats->{days}}++;
+                $stats->{day_times}[$stats->{days}][int($hour/6)]++;
+                $stats->{day_lines}->[$stats->{days}]++;
 
                 $stats->{actions}{$nick}++;
                 push @{ $lines->{actionlines}{$nick} }, $line;
@@ -437,7 +464,7 @@ sub _parse_file
                 $stats->{lastvisited}{$nick} = $stats->{days};
                 $stats->{line_times}{$nick}[int($hour/6)]++;
 
-                if ($saying =~ /$self->{violentwords_regexp}/) {
+                if ($self->{violentwords_regexp} and $saying =~ /$self->{violentwords_regexp}/) {
                     my $victim;
                     unless ($victim = is_nick($2)) {
                         foreach my $trynick (split(/\s+/, $3)) {
@@ -455,12 +482,20 @@ sub _parse_file
                     }
                 }
 
+                if ($saying =~ /$self->{chartsregexp}/i) {
+                    $self->_charts($1, $nick);
+                }
 
                 $stats->{lengths}{$nick} += length($saying);
 
+                if (my $s = $self->{users}->{sex}{$nick}) {
+                    $stats->{sex_lines}{$s}++;
+                    $stats->{sex_line_times}{$s}[int($hour/6)]++;
+                }
+
                 _parse_words($stats, $saying, $nick, $self->{ignorewords_regexp}, $hour);
-            }
-        }
+            } # ignored
+        } # action lines
 
         # Match *** lines.
         elsif (($hashref = $self->{parser}->thirdline($line, $.)) and $hashref->{nick}) {
@@ -480,22 +515,20 @@ sub _parse_file
             $newjoin  = $hashref->{newjoin};
             $newnick  = $hashref->{newnick};
 
-            if ($hour < $state->{oldtime}) {
+            if ($hour < $stats->{oldtime}) {
+                $stats->{firsttime} = $hour if $stats->{oldtime} == 24; # save stamp for merging
                 $stats->{days}++;
-                $stats->{day_times}{$stats->{days}}[0]=0;
-                $stats->{day_times}{$stats->{days}}[1]=0;
-                $stats->{day_times}{$stats->{days}}[2]=0;
-                $stats->{day_times}{$stats->{days}}[3]=0;
-                $stats->{day_lines}{$stats->{days}}=0;
+                @{$stats->{day_times}[$stats->{days}]} = (0, 0, 0, 0);
+                $stats->{day_lines}->[$stats->{days}] = 0;
             }
 
-            $state->{oldtime} = $hour;
+            $stats->{oldtime} = $hour;
 
             if (!is_ignored($nick)) {
                 # Timestamp collecting
                 $stats->{times}{$hour}++;
-                $stats->{day_times}{$stats->{days}}[int($hour/6)]++;
-                $stats->{day_lines}{$stats->{days}}++;
+                $stats->{day_times}[$stats->{days}][int($hour/6)]++;
+                $stats->{day_lines}->[$stats->{days}]++;
 
                 $stats->{lastvisited}{$nick} = $stats->{days};
 
@@ -507,7 +540,13 @@ sub _parse_file
                     }
 
                 } elsif (defined($newtopic) && $newtopic ne '') {
-                    _topic_change($stats, $newtopic, $nick, $hour, $min);
+                    push @{$stats->{topics}}, {
+                        topic => $newtopic,
+                        nick  => $nick,
+                        hour  => $hour,
+                        min   => $min,
+                        days  => $stats->{days},
+                    };
 
                 } elsif (defined($newmode)) {
                     _modechanges($stats, $newmode, $nick);
@@ -522,33 +561,27 @@ sub _parse_file
                     checkname($nick, $newnick, $stats) if ($self->{cfg}->{showmostnicks});
                 }
             }
+        } # *** lines
+
+        unless ($stats->{parsedlines} % 10000) { # keep only recent quotes to save memory
+            $self->_trim_lines($lines);
+        }
+    } # while(my $line = <LOGFILE>)
+
+    $self->_trim_lines($lines);
+
+    my $wordcount = sqrt(sqrt(keys %{$stats->{wordcounts}})); # remove less frequent words
+    foreach my $word (keys %{$stats->{wordcounts}}) {
+        if ($stats->{wordcounts}->{$word} < $wordcount) {
+            delete $stats->{wordcounts}->{$word};
+            delete $stats->{wordnicks}->{$word};
+            delete $stats->{word_upcase}->{$word};
         }
     }
 
     $stats->{totallines} = $.;
 
     close(LOGFILE);
-
-    print "Finished analyzing log, $stats->{days} days, $stats->{parsedlines} lines total\n"
-        unless ($self->{cfg}->{silent});
-}
-
-sub _topic_change
-{
-    my $stats = shift;
-    my $newtopic = shift;
-    my $nick = shift;
-    my $hour = shift;
-    my $min = shift;
-
-    my $tcount = 0;
-    if (defined $stats->{topics}) {
-        $tcount = @{ $stats->{topics} };
-    }
-    $stats->{topics}[$tcount]{topic} = $newtopic;
-    $stats->{topics}[$tcount]{nick}  = $nick;
-    $stats->{topics}[$tcount]{hour}  = $hour;
-    $stats->{topics}[$tcount]{min}   = $min;
 }
 
 sub _modechanges
@@ -581,7 +614,7 @@ sub _modechanges
 
 sub _parse_words
 {
-    my ($stats, $saying, $nick, $ignorewords, $hour) = @_;
+    my ($stats, $saying, $nick, $ignorewords_regexp, $hour) = @_;
     # Cache time of day
     my $tod = int($hour/6);
 
@@ -589,7 +622,7 @@ sub _parse_words
         $stats->{words}{$nick}++;
         $stats->{word_times}{$nick}[$tod]++;
         # remove uninteresting words
-        next if $ignorewords and $word =~ m/$ignorewords/i;
+        next if $ignorewords_regexp and $word =~ m/$ignorewords_regexp/i;
 
         # ignore contractions
         next if ($word =~ m/'.{1,2}$/o);
@@ -597,14 +630,38 @@ sub _parse_words
         # Also ignore stuff from URLs.
         next if ($word =~ m/^https?$|^\/\//o);
 
-	# uniquify nicks
-	if (my $realnick = is_nick($word)) {
-            $stats->{wordcounts}{$realnick}++;
-            $stats->{wordnicks}{$realnick} = $nick;
-	} else {
-            $stats->{wordcounts}{$word}++;
-            $stats->{wordnicks}{$word} = $nick;
-	}
+        my $lcword = lc $word;
+        $stats->{wordcounts}{$lcword}++;
+        $stats->{wordnicks}{$lcword} = $nick;
+        $stats->{word_upcase}{$lcword} ||= $word; # remember first-seen case
+    }
+}
+
+sub _charts
+{
+    my ($stats, $Song, $nick) = @_;
+    $Song =~ s/_/ /g;
+    $Song =~ s/\d+ ?- ?//;
+    $Song =~ s/\.mp3//g;
+    $Song =~ s/ \.\.\.$//;
+    $Song =~ s/ [^\w]+$//;
+    my $song = lc $Song;
+    $stats->{word_upcase}{$song} = $Song;
+    $stats->{chartcounts}{$song}++;
+    $stats->{chartnicks}{$song} = $nick;
+}
+
+sub _trim_lines
+{
+    my ($self, $lines) = @_;
+
+    foreach my $n (keys %{$lines->{sayings}}) {
+        my $x = @{$lines->{sayings}->{$n}};
+        splice(@{$lines->{sayings}->{$n}}, 0, ($x - 15)) if ($x > 30);
+    }
+    foreach my $n (keys %{$lines->{actionlines}}) {
+        my $x = @{$lines->{actionlines}->{$n}};
+        splice(@{$lines->{actionlines}->{$n}}, 0, ($x - 15)) if ($x > 30);
     }
 }
 
@@ -614,34 +671,37 @@ sub _pick_random_lines
 
     foreach my $key (keys %{ $lines }) {
         foreach my $nick (keys %{ $lines->{$key} }) {
-            $stats->{$key}{$nick} = $self->_random_line($stats, $lines, $key, $nick, 0);
+            $stats->{$key}{$nick} = $self->_random_line($lines, $key, $nick);
         }
     }
 }
 
 sub _random_line
 {
-    my ($self, $stats, $lines, $key, $nick, $count) = @_;
-    my $random = ${ $lines->{$key}{$nick} }[rand@{ $lines->{$key}{$nick} }];
-    if ($self->{cfg}->{noignoredquotes} && $random =~ /$self->{ignorewords_regexp}/i) {
-        return '' if ($count > 20);
-        return $self->_random_line($stats, $lines, $key, $nick, ++$count);
-    } else {
-        return $random;
-    }
+    my ($self, $lines, $key, $nick) = @_;
+    my $count = 0;
+    my $random;
+    #warn "$nick did not say anything" unless @{ $lines->{$key}{$nick} };
+    do {
+        $random = ${ $lines->{$key}{$nick} }[rand @{ $lines->{$key}{$nick} }];
+        return '' if ++$count > 20;
+    } while ($self->{cfg}->{noignoredquotes} and $self->{ignorewords_regexp} and $random =~ /$self->{ignorewords_regexp}/i);
+    return $random;
 }
 
 sub _uniquify_nicks {
     my ($stats) = @_;
 
     foreach my $word (keys %{ $stats->{wordcounts} }) {
-        if (my $realnick = is_nick($word)) {
-	    # The lc() is an attempt at being case insensitive.
-	    if (lc($realnick) ne lc($word)) {
-	        $stats->{wordcounts}{$realnick} += $stats->{wordcounts}{$word};
-	        delete $stats->{wordcounts}{$word};
-	        delete $stats->{wordnicks}{$word};
-	    }
+        if (my $realnick = lc(is_nick($word))) {
+            if ($realnick ne $word) { # word is always lc
+                $stats->{wordcounts}{$realnick} += $stats->{wordcounts}{$word};
+                $stats->{wordnicks}{$realnick} ||= $stats->{wordnicks}{$word};
+                $stats->{word_upcase}{$realnick} ||= $stats->{word_upcase}{$word};
+                delete $stats->{wordcounts}{$word};
+                delete $stats->{wordnicks}{$word};
+                delete $stats->{word_upcase}{$word};
+            }
         }
     }
 }
@@ -667,7 +727,7 @@ sub _strip_mirccodes
 
 sub checkname {
     # This function tracks nickchanges and puts them all in a hash->array,
-    # so we can show all nicks that I user had later (only works properly
+    # so we can show all nicks that a user had later (only works properly
     # when nicktracking is enabled)
     my ($nick, $newnick, $stats) = @_;
 
@@ -683,10 +743,143 @@ sub _adjusttimeoffset
         $hour += $self->{cfg}{timeoffset};
         $hour = $hour % 24;
     }
-     
+
     return sprintf('%02d', $hour);
 }
 
+sub _read_cache
+{
+    my ($self, $statsref, $linesref, $logfile) = @_;
+    my $mtime = (stat $logfile)[9];
+    my $cachefile = $logfile;
+    $cachefile =~ s/[^\w-]/_/g;
+    $cachefile = "$self->{cfg}->{cachedir}/$cachefile";
+
+    return undef unless -e $cachefile;
+    open C, $cachefile or die "$cachefile: $!";
+    local $/;
+    my $str = <C>;
+    close C;
+
+    my ($stats, $lines);
+    eval $str;
+
+    return undef if $stats->{version} and $stats->{version} ne $self->{cfg}->{version};
+    return undef unless $stats->{logfile} eq $logfile; # the name might be ambigous
+    return undef if $stats->{logfile_mtime} != $mtime; # file has changed
+
+    print "cached, " unless $self->{cfg}->{silent};
+    $$statsref = $stats;
+    $$linesref = $lines;
+
+    return 1;
+}
+
+sub _update_cache
+{
+    my ($self, $stats, $lines, $logfile) = @_;
+    my $mtime = (stat $logfile)[9];
+    my $cachefile = $logfile;
+    $cachefile =~ s/[^\w-]/_/g;
+    $cachefile = "$self->{cfg}->{cachedir}/$cachefile";
+
+    #print "Writing cache $cachefile...";
+
+    $stats->{logfile} = $logfile;
+    $stats->{logfile_mtime} = $mtime;
+
+    unless (open C, ">$cachefile") {
+            die "$cachefile: $!";
+    }
+    $stats->{version} = $self->{cfg}->{version};
+    print C Data::Dumper->Dump([$stats], ["stats"]);
+    print C Data::Dumper->Dump([$lines], ["lines"]);
+    close C;
+}
+
+sub _merge_stats
+{
+    my ($self, $stats, $s) = @_;
+
+    my $days_offset = $stats->{days};
+    my $days_rollover = $stats->{oldtime} > $s->{firsttime};
+    $stats->{days} += $s->{days} - 1 + $days_rollover;
+
+    foreach my $key (keys %$s) {
+        #print "$key -> $s->{$key}\n";
+        if ($key =~ /^(logfile|firsttime|days|version)/) { # don't merge these
+            next;
+        } elsif ($key =~ /^(oldtime|lastnick|lastnormal|monocount)$/) { # {key} = int/str: copy
+            $stats->{$key} = $s->{$key};
+        } elsif ($key =~ /^(parsedlines|totallines)$/) { # {key} = int: add
+            $stats->{$key} += $s->{$key};
+        } elsif ($key =~ /^(wordnicks|word_upcase|urlnicks|chartnicks)$/) { # {key}->{} = str: copy
+            foreach my $subkey (keys %{$s->{$key}}) {
+                $stats->{$key}->{$subkey} = $s->{$key}->{$subkey};
+            }
+        } elsif ($key =~ /^(nicks|karma)$/) { # {key}->{}->{} = str: copy
+            foreach my $subkey (keys %{$s->{$key}}) {
+                foreach my $value (keys %{$s->{$key}->{$subkey}}) {
+                    $stats->{$key}->{$subkey}->{$value} = $s->{$key}->{$subkey}->{$value};
+                }
+            }
+        } elsif ($key =~ /^(word|line|sex_line)_times$/) { # {key}->{}->[] = int: add
+            foreach my $subkey (keys %{$s->{$key}}) {
+                foreach my $pos (0 .. @{$s->{$key}->{$subkey}} - 1) {
+                    $stats->{$key}->{$subkey}->[$pos] += $s->{$key}->{$subkey}->[$pos]
+                        if $s->{$key}->{$subkey}->[$pos];
+                }
+            }
+        } elsif ($key eq 'lastvisited') { # {key}->{} = int: copy
+            foreach my $nick (keys %{$s->{lastvisited}}) {
+                $stats->{lastvisited}->{$nick} =
+                    $days_offset + $s->{lastvisited}->{$nick} - 1 + $days_rollover;
+            }
+        } elsif ($s->{$key} =~ /^HASH/) { # {key}->{} = int: add
+            foreach my $subkey (keys %{$s->{$key}}) {
+                die "$key -> $subkey" unless $s->{$key}->{$subkey} =~ /^\d+/; # assert
+                $stats->{$key}->{$subkey} += $s->{$key}->{$subkey};
+            }
+        } elsif ($key =~ /^topics$/) { # {key}->[] = topic hash: append
+            push @{$stats->{$key}}, map {
+                my %a = %$_; $a{days} += $days_offset; \%a; # make new hash
+            } @{$s->{$key}};
+        } elsif ($key =~ /^day_lines$/) { # {key}->[] = int: append
+            my @list = @{$s->{day_lines}};
+            die if splice @list, 0, 1; # first element is always undef
+            unless ($days_rollover) {
+                $stats->{day_lines}->[$days_offset] += splice @list, 0, 1;
+            }
+            push @{$stats->{day_lines}}, @list;
+        } elsif ($key =~ /^day_times$/) { # {key}->[]->[] = int: append outer list
+            my @list = @{$s->{day_times}};
+            die if splice @list, 0, 1;
+            if (not $days_rollover) {
+                my @first = @{splice @list, 0, 1};
+                foreach my $pos (0 .. @first - 1) {
+                    $stats->{day_times}[$days_offset][$pos] += $first[$pos];
+                }
+            }
+            push @{$stats->{day_times}}, map { my @a = @$_; \@a; } @list;
+        } else {
+            die "unknown key format $key -> $s->{$key}";
+        }
+    }
+}
+
+sub _merge_lines
+{
+    my ($self, $lines, $l) = @_;
+
+    foreach my $key (keys %$l) { # sayings, actionlines, etc.
+        foreach my $subkey (keys %{$l->{$key}}) {
+            push @{$lines->{$key}->{$subkey}}, @{$l->{$key}->{$subkey}};
+            my $x = @{$lines->{$key}->{$subkey}};
+            splice(@{$lines->{$key}->{$subkey}}, 0, ($x - 15)) if ($x > 30);
+        }
+    }
+}
+
 1;
 
 __END__
@@ -704,7 +897,7 @@ C<Pisg::Parser::Logfile> parses a logfile using the configuration variables set
     use Pisg::Parser::Logfile;
 
     $analyzer = new Pisg::Parser::Logfile(
-        cfg => $cfg,
+        { cfg => $self->{cfg}, users => $self->{users} }
     );
 
 =head1 CONSTRUCTOR
@@ -713,11 +906,9 @@ C<Pisg::Parser::Logfile> parses a logfile using the configuration variables set
 
 =item new ( [ OPTIONS ] )
 
-This is the constructor for a new Pisg::Parser::Logfile object. C<OPTIONS> are passed in a hash like fashion using key and value pairs.
+This is the constructor for a new Pisg::Parser::Logfile object.
 
-Possible options are:
-
-B<cfg> - hashref containing configuration variables, created by the Pisg module.
+The first option must be a reference to a hash containing the cfg and users structures.
 
 =back
 
@@ -727,7 +918,8 @@ Morten Brix Pedersen <morten@wtf.dk>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2001 Morten Brix Pedersen. All rights resereved.
+Copyright (C) 2001-2005 Morten Brix Pedersen. All rights resereved.
+Copyright (C) 2003-2005 Christoph Berg <cb@df7cb.de>.
 This program is free software; you can redistribute it and/or modify it
 under the terms of the GPL, license is included with the distribution of
 this file.
